#!/usr/bin/python
#
# RASP: RASP Aids Song Production
# Version 1.5.1
# Copyright 2012, Brian Hilmers
# License: OSL 3.0
# Python 2.6
#
# This is a collection of three handy tools for electronic music production:
# Tap Tempo, a Song Time calculator, and a Delay/Hertz calculator for each
# note division. These are the tools I use most when writing, all in one place!

from time import time
from Tkinter import *
import tkMessageBox
import tkFont

tap_time = time()
previous_time = time()
tap_count = 0


def tap(x):
    # This function is a complete rewrite after 2.1, modeled
    # after Richard Reel's BPM Tool on http://www.all8.com
    global previous_time 
    global tap_time
    global tap_count
    current_time = time()
    bpm = 0
    
    if (current_time - previous_time) > 2:
        tap_count = 0
    if tap_count == 0:
        tap_time = time()
        tap_count = 1
    else:
        bpm = round(60 * tap_count / (current_time - tap_time), 1)
        tap_count += 1
        
    previous_time = current_time
    input_bpm.delete(0, END)
    input_bpm.insert(0, bpm)

def calculate():
    # This is a whole bunch of convering input to
    # other numbers and smashing them together.
    validate()
    # Transpose the beat value
    if int(input_beat_value.get()) == 1: beat_value = 4
    elif int(input_beat_value.get()) == 2: beat_value = 2
    elif int(input_beat_value.get()) == 4: beat_value = 1
    elif int(input_beat_value.get()) == 8: beat_value = .5
    elif int(input_beat_value.get()) == 16: beat_value = .25
    elif int(input_beat_value.get()) == 32: beat_value = .125
    elif int(input_beat_value.get()) == 64: beat_value = .0625
    
    seconds_per_beat = 60 / float(input_bpm.get())
    seconds_per_measure = (seconds_per_beat * 
             float(input_beats_per_measure.get()) * beat_value)
    
    # Song time calculation
    song_time = (seconds_per_measure * float(input_total_measures.get())) / 60
    # Split into minutes and seconds
    song_time_minutes = int(song_time)
    song_time_remainder = song_time - song_time_minutes
    song_time_seconds = int(round(60 * song_time_remainder))
    
    # Print out to song time label
    x = "%i:%02i" % (song_time_minutes, song_time_seconds)
    song_time_output.set(x)
    
    
    # Print out to the delay/hertz labels
    if delay_range.get() == 0:
        dcalc_1.set(populate(.0625, "16/1"))
        dcalc_2.set(populate(.125, "8/1"))
        dcalc_3.set(populate(.25, "4/1"))
        dcalc_4.set(populate(.3333, "3/1"))
        dcalc_5.set(populate(.5, "2/1"))
        dcalc_6.set(populate(1, "1/1"))
    else:    
        dcalc_1.set(populate(2, "1/2"))
        dcalc_2.set(populate(4, "1/4"))
        dcalc_3.set(populate(8, "1/8"))
        dcalc_4.set(populate(16, "1/16"))
        dcalc_5.set(populate(32, "1/32"))
        dcalc_6.set(populate(64, "1/64"))

def validate():
    # Check to see if all fields are valid.
    try:
        if float(input_bpm.get()) <= 0:
            input_bpm.delete(0, END)
            input_bpm.insert(0, 1)
    except:
        input_bpm.delete(0, END)
        input_bpm.insert(0, 120)
    
    try:
        if float(input_beats_per_measure.get()) <= 0:
            input_beats_per_measure.delete(0, END)
            input_beats_per_measure.insert(0, 1)
    except:
        input_beats_per_measure.delete(0, END)
        input_beats_per_measure.insert(0, 4)
    
    try:
        x = float(input_beat_value.get())
        if not (x == 1 or x == 2 or x == 4 or
                x == 8 or x == 16 or x ==32 or x == 64):
            tkMessageBox.showwarning("Note Value", "Use the following:\n\n" +
            " (1)  Whole note\n (2)  Half note\n (4)  Quarter note\n" +
            " (8)  Eighth note\n(16) Sixteenth note\n" +
            "(32) Thirty-second note\n(64) Sixty-fourth note", icon="error")
            input_beat_value.delete(0, END)
            input_beat_value.insert(0, "4")
            input_beat_value.focus_set()
    except:
        input_beat_value.delete(0, END)
        input_beat_value.insert(0, "4")

    try:
        if float(input_total_measures.get()) < 0:
            input_total_measures.delete(0, END)
            input_total_measures.insert(0, 0)
    except:
        input_total_measures.delete(0, END)
        input_total_measures.insert(0, 0)

def populate(divisor, divisor_string):
    # The delay and hertz calculation. Get a few basic values, then put the
    # modified versions right into a nicely formated string. Combine all the
    # strings for easy output to the tkLabel. Note: "divisor" is really
    # "input_beat_value" from above, minus whole notes.
    milliseconds = (((60 * 1000) / float(input_bpm.get())) * 
                   (float(input_beat_value.get()) / divisor))
    seconds_per_beat = 60 / float(input_bpm.get())
    hertz = seconds_per_beat * (float(input_beat_value.get()) / divisor);
    
    line1 = "%s. % 6i ms % 6.2f Hz\n" % (divisor_string, (milliseconds * 1.5),
                                          (1 / (hertz * 1.5)))
    line2 = "%s  % 6i ms % 6.2f Hz\n" % (divisor_string, (milliseconds * 1.0),
                                          (1 / (hertz * 1.0)))
    line3 = "%st % 6i ms % 6.2f Hz" % (divisor_string, (milliseconds * .67),
                                        (1 / (hertz * .67)))
    string = line1 + line2 + line3
    return string

def note2freq(self):
    # Calcualte frequency from a note value
    note_name = ""
    octave = slide_octave.get()
    
    if slide_note.get() == 1: x = 3; note_name = "C"
    elif slide_note.get() == 2: x = 4; note_name = "C#"
    elif slide_note.get() == 3: x = 5; note_name = "D"
    elif slide_note.get() == 4: x = 6; note_name = "D#"
    elif slide_note.get() == 5: x = 7; note_name = "E"
    elif slide_note.get() == 6: x = 8; note_name = "F"
    elif slide_note.get() == 7: x = 9; note_name = "F#"
    elif slide_note.get() == 8: x = 10; note_name = "G"
    elif slide_note.get() == 9: x = 11; note_name = "G#"
    elif slide_note.get() == 10: x = 0; note_name = "A"; octave = octave + 1
    elif slide_note.get() == 11: x = 1; note_name = "A#"; octave = octave + 1
    elif slide_note.get() == 12: x = 2; note_name = "B"; octave = octave + 1

    y = slide_concert.get() / float(2 ** (5 - octave))
    frequency = 2 ** (x / float(12)) * y
    # The following alternate formula is taken from:
    # http://en.wikipedia.org/wiki/
    # Cent_%28music%29#Piecewise_linear_approximation
    # c = frequency * (1 + .0005946 * cents)
    frequency_with_cents = frequency * 2 ** (slide_cent.get() / float(1200))
    
    # Show selected note value, frequency
    concert_label = str(slide_concert.get())
    octave_label = str(slide_octave.get())
    cent_label = str(slide_cent.get())
    
    label_note = note_name + octave_label + "." + cent_label
    note_value_display.set(label_note)
    freq_value_display.set(round(frequency_with_cents, 2))
    concert_value_display.set("Concert A (" + concert_label + ")")
    

def about():
    about_string = (
    "RASP Aids Song Production\n" +
    "Version 1.5.2\n\n" +
    "Brian Hilmers\n" +
    "Copyright: 2012\n" +
    "License: OSL 3.0"
    )
    top = Toplevel()
    top.grab_set()
    top.geometry()
    top.resizable(0, 0)
    top.title("RASP - About")
    label = Label(top, text=about_string, padx=20, pady=10)
    label.pack()


root = Tk()

# Info Frame: Tap tempo, all inputs, calcualte, song time output
infoframe = Frame(pady=5)
infoframe.pack(anchor=W, padx=5)

# Column 1 & 2: Song details
input_bpm = Entry(infoframe, width=5, justify=CENTER)
input_bpm.insert(0, "120")
input_bpm.focus_set()
input_bpm.grid(row=0, column=1, padx=5)
label_beats_per_minute = Label(infoframe, text="BPM")
label_beats_per_minute.grid(row=0, column=2, sticky=W)

input_beats_per_measure = Entry(infoframe, width=5, justify=CENTER)
input_beats_per_measure.insert(0, "4")
input_beats_per_measure.grid(row=1, column=1)
input_beat_value = Entry(infoframe, width=5, justify=CENTER)
input_beat_value.insert(0, "4")
input_beat_value.grid(row=2, column=1)
label_beats_per_measure = Label(infoframe, text="Time\nSignature", justify=LEFT)
label_beats_per_measure.grid(row=1, column=2, rowspan=2, sticky=W)

input_total_measures = Entry(infoframe, width=5, justify=CENTER)
input_total_measures.insert(0, "0")
input_total_measures.grid(row=3, column=1)
label_total_measures = Label(infoframe, text="Measures")
label_total_measures.grid(row=3, column=2, sticky=W)

delay_range = IntVar()
radio_long = Radiobutton(infoframe, indicatoron=0,
                          width=5, text="long", variable=delay_range, value=0)
radio_long.grid(row=4, column=1)
radio_short = Radiobutton(infoframe, indicatoron=0,
                           width=5, text="short", variable=delay_range, value=1)
radio_short.grid(row=5, column=1)
radio_short.select()
label_delay_selector = Label(infoframe, text="Delay\nLength", justify=LEFT)
label_delay_selector.grid(row=4, column=2, rowspan=2, sticky=W)

# Column 3: Song time display, calculate button
label_song_time_frame = LabelFrame(infoframe, text="Song Time", labelanchor=N)
label_song_time_frame.grid(row=0, column=3, rowspan=3, padx=15)
song_time_output = StringVar()
song_time_output.set("0:00")
large_font = tkFont.Font(size=16, weight="bold")
label_song_time = Label(label_song_time_frame, textvariable=song_time_output,
                      font=large_font, width=8, height=2).pack()

button_calculate = Button(infoframe, text="Calculate",
                          command=calculate, width=12)
button_calculate.grid(row=3, column=3, rowspan=2)

label_tap_tempo = Label(infoframe, text="Tap space for tempo")
label_tap_tempo.grid(row=5, column=3)


# Added the logo since I have some extra space. Looks fun, haha!
rasp_image='''\
R0lGODlhgACAAIABAAAAAP///yH+EUNyZWF0ZWQgd2l0aCBHSU1QACH5BAEKAAEALAAAAACAAIAA
AAL+jI+py+0Po5y02ouz3rz7D4biSIbAiablyk7pqzJoS4Oxch/wXPcX34jtYL5iBOjYyZBGUg53
egwB0qhB2dQModQkFjJ9ZX/hLjNR9n6vz7GaK1ZPl3F61/0OntFlq26eh7fU0XfTV3UniLBnUehY
t7im6Kf1aElZOElJeHkYcIm3WdIJ+pdYJFpDmhZ5qpratAr5CbtSG0oKZXSraOr5h8rbC0fU6iOc
AWiSScPIYbmsbOv6IesRxuK8od3aRiYpgvxNDbZJXl48wp0sPriIAV5NdXde4W2axAc/Kz8P/Axr
ywJX9RC1s3dqHTpq2HS9Y3fQRUSJDFmhuZgvkI3+ghAJAkn3UAexIxMROUnFQyBGkfgQLjspSkg8
HLQ46rEpQeG3gVak3QSIsxzMimZUZuQU1AtMOmxmTkvKcynTmvxeQdWnrtbHqj10cslaL8e9rl7/
hePlc0zZmidJcs1StmTOq8OMuR1Ft27LKtny6qWFrsXbvzwLyh13GJdGq4nh3mrc0a/jc2ujSd6F
svJZyGQTgpxMuKLdz5j/Em3abXDfuuR6emxIuFnrt45i47XDsZRtTjeT5nqpVg5dWX7HCg47T7Mv
4plVT1NK9RpzZrH4+gI7nfO+xc431nbzuLn23RTQIldOnsziwuPTTy1Msb37SL0Rop/P/vVE0vj+
xxG7GoVx/QW2nkndDRiVUOUlFx2CC57GxoOAHejgHkgcFGBRbDlI4HX0zcUgYBty+F5THkkkYooC
9neGhTjN4Foi/LEY14vJFaXfiqbNyIeNKgZkUWwqqCZOOuvAphcTXOmUoV3xXYYdbp6BmGKEjci3
zZFIKnXfcprUeCCMULKFpX1lDtKlTGOa2aV9I1554ZlUpXWclWwS1WaPPB6n4YPOfKfeMH604xxx
JLEG1Z572aNgL4TqaFZWggqj6Fe3OWoYhR9eqomUEK2G6S/bPYWoUZ/qSedPSeb5IXVXkghnkCs1
CiuITllpk5yrYtinSbXeRaWd1v3qa4de6Rpz6kJOdkgsfMItS2uzDuUnI5jSlmiikmsh2+lyMUG6
6LX4mJqtm+L+9ye4DZ6bWnjqqksikRTCGy8ylZbLboIGUaopsfxyc2u+AI8VcL7CYpstq7CaV23B
Bk8LX6oPHyplvxNTu+XFWUqscccefwxyyPgVAAA7
'''
rasp_image = PhotoImage(data=rasp_image)
rasp_logo = Label(infoframe, image=rasp_image)
rasp_logo.image = rasp_image
rasp_logo.grid(row=0, column=4, rowspan=6)


# Delay Frame: Delay calcuation output --> populate()
delayframe = Frame()
delayframe.pack()

# Column 0
label_1_frame = LabelFrame(delayframe)
label_1_frame.grid(row=1, column=0)
dcalc_1 = StringVar()
dcalc_1.set("\n\n")
label_1 = Label(label_1_frame, textvariable=dcalc_1, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_2_frame = LabelFrame(delayframe)
label_2_frame.grid(row=2, column=0)
dcalc_2 = StringVar()
dcalc_2.set("\n\n")
label_2 = Label(label_2_frame, textvariable=dcalc_2, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_3_frame = LabelFrame(delayframe)
label_3_frame.grid(row=3, column=0)
dcalc_3 = StringVar()
dcalc_3.set("\n\n")
label_3 = Label(label_3_frame, textvariable=dcalc_3, justify=RIGHT,
                width=28, font="TkFixedFont").pack()

# Column 1
label_4_frame = LabelFrame(delayframe)
label_4_frame.grid(row=1, column=1)
dcalc_4 = StringVar()
dcalc_4.set("\n\n")
label_4 = Label(label_4_frame, textvariable=dcalc_4, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_5_frame = LabelFrame(delayframe)
label_5_frame.grid(row=2, column=1)
dcalc_5 = StringVar()
dcalc_5.set("\n\n")
label_5 = Label(label_5_frame, textvariable=dcalc_5, justify=RIGHT,
                width=28, font="TkFixedFont").pack()
label_6_frame = LabelFrame(delayframe)
label_6_frame.grid(row=3, column=1)
dcalc_6 = StringVar()
dcalc_6.set("\n\n")
label_6 = Label(label_6_frame, textvariable=dcalc_6, justify=RIGHT,
                width=28, font="TkFixedFont").pack()

# Note-to-Frequecny Frame: Independent of calulate function --> note2freq()
n2f_frame = Frame()
n2f_frame.pack()

slide_concert = Scale(n2f_frame, from_=435, to=445, label="Concert A",
                        length=240, width=10, orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_concert.grid(row=0, column=0)
slide_note = Scale(n2f_frame, from_=1, to=12, label="Note",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_note.grid(row=1, column=0)
slide_octave = Scale(n2f_frame, from_=0, to=8, label="Octave",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_octave.grid(row=2, column=0)
slide_cent = Scale(n2f_frame, from_=-50, to=50, label="Cents",
                        length=240, width=10,  orient=HORIZONTAL,
                        showvalue=0, command=note2freq)
slide_cent.grid(row=3, column=0)

# Set default values to A4.0 @ 440Hz
slide_concert.set(440)
slide_note.set(10)
slide_octave.set(4)
slide_cent.set(0)

note_value_display = StringVar()
concert_value_display = StringVar()
concert_value_display.set(slide_concert.get())
note_value_frame = LabelFrame(n2f_frame, text="Note", labelanchor=N)
note_value_frame.grid(row=0, column=1, rowspan=2, padx=2)
note_value_display_label = Label(note_value_frame, font=large_font, width=10,
                                 height=1, textvariable=note_value_display)
note_value_display_label.pack()
concert_value_display_label = Label(note_value_frame,
                                    textvariable=concert_value_display)
concert_value_display_label.pack()

freq_value_display = StringVar()
freq_value_frame = LabelFrame(n2f_frame, text="Frequency (Hz)", labelanchor=N)
freq_value_frame.grid(row=2, column=1, rowspan=2, padx=2)
freq_value_display_label = Label(freq_value_frame, font=large_font, width=10,
                                 height=2, textvariable=freq_value_display)
freq_value_display_label.pack()

# Reserved for later.
bottom_frame = Frame()
bottom_frame.pack(pady=5)
garbage_frame = LabelFrame(bottom_frame)
garbage_frame.grid(row=0, column=0)
garbage_label = Label(garbage_frame, text="",width=45)
garbage_label.pack()
button_about = Button(bottom_frame, text="?", command=about)
button_about.grid(row=0, column=1)

# Launch the GUI, bind spacebar to main window.
root.bind("<space>", tap)
root.title("RASP")
root.geometry("425x490+50+50")
root.resizable(0, 0)
root.mainloop()
